@@ -11,14 +11,34 @@ from pathlib import Path
 from EasyHTTPServerAJM import EasyHTTPLogger
 
 
-# TODO: needs logging support
 class EasyHTTPServer:
     """
-    Basic class-based HTTP server for file sharing.
+    A simple HTTP server for serving files with directory listing functionality.
 
-    - Serves files from a given directory (default: current directory).
-    - Uses Python's built-in http.server.SimpleHTTPRequestHandler.
-    - Intended for quick, local file sharing on trusted networks.
+    The EasyHTTPServer class provides a lightweight and configurable HTTP server
+    for file-sharing purposes. It supports directory browsing, customizable
+    handlers, and file-serving options. The server is designed to be started using
+    command-line arguments or programmatically.
+
+    :ivar logger: Logger instance to track server events and errors.
+    :type logger: EasyHTTPLogger.logger
+    :ivar html_template_path: Path to an optional HTML template for custom directory
+        listing pages. Defaults to None if not provided.
+    :type html_template_path: str, optional
+    :ivar directory: Path of the directory to serve. Defaults to the current
+        directory if not passed.
+    :type directory: Path
+    :ivar host: Host/IP address to bind the server to. Defaults to "0.0.0.0",
+        which binds to all network interfaces.
+    :type host: str
+    :ivar port: Port number to run the server. Defaults to 8000.
+    :type port: int
+    :ivar handler_class: Request handler class to use for managing HTTP requests.
+        Can be customized; otherwise, defaults to PrettyDirectoryHandler.
+    :type handler_class: type
+    :ivar start_time: Timestamp indicating when the server started. None if the
+        server has not started yet.
+    :type start_time: datetime, optional
     """
 
     DEFAULT_HANDLER_CLASS = PrettyDirectoryHandler#SimpleHTTPRequestHandler
@@ -44,12 +64,38 @@ class EasyHTTPServer:
         self._httpd: Optional[TCPServer] = None
         self.start_time: Optional[datetime] = None
 
+    @classmethod
+    def __version__(cls):
+        try:
+            return str(__version__)
+        except NameError:
+            return ' unknown'
+
     @classmethod
     def from_cli(cls) -> "EasyHTTPServer":
         """Create an EasyHTTPServer instance using command-line arguments."""
         args = cls._parse_args()
         return cls(directory=args.directory, host=args.host, port=args.port)
 
+    @classmethod
+    def get_welcome_string(cls) -> str:
+        return f"{cls.__name__} v{cls.__version__()}"
+
+    @property
+    def serving_info_string(self) -> str:
+        # noinspection HttpUrlsUsage
+        return f"Serving directory {self.directory.resolve()} at http://{self.host}:{self.port}"
+
+    def _log_all_basic_server_info(self, **kwargs):
+        print_msg = kwargs.pop("print_msg", True)
+        # noinspection PyArgumentList
+        self.logger.info(self.__class__.get_welcome_string(), print_msg=print_msg, **kwargs)
+        # noinspection HttpUrlsUsage
+        # noinspection PyArgumentList
+        self.logger.info(self.serving_info_string, print_msg=print_msg, **kwargs)
+        if print_msg:
+            print("Press Ctrl+C to stop.")
+
     @property
     def runtime(self):
         if self.start_time:
@@ -84,27 +130,47 @@ class EasyHTTPServer:
         return parser.parse_args()
 
     def _handler_factory(self, request, client_address, server):
-        return self.handler_class(request,
-                                  client_address,
-                                  server,
-                                  directory=self.directory,
-                                  logger=self.logger,
-                                  html_template_path=self.html_template_path)
+        """
+        Creates an instance of the specified handler class with the provided parameters.
 
-    def start(self) -> None:
+        This method initializes and returns a handler object using
+        the `handler_class` attribute. It passes the necessary arguments
+        such as request, client address, server, and additional
+        parameters like directory, logger, and HTML template path
+        to the handler's constructor.
+
+        :param request: The incoming client request to be handled.
+        :param client_address: The address of the client sending the request.
+        :param server: The server instance managing the request.
+        :return: An instance of the handler class initialized with the given parameters.
+        :rtype: handler_class
+        """
+        try:
+            return self.handler_class(request,
+                                      client_address,
+                                      server,
+                                      directory=self.directory,
+                                      logger=self.logger,
+                                      html_template_path=self.html_template_path)
+        except Exception as e:
+            self.logger.critical(f"Failed to create handler: {e}")
+            self.err_stop()
+
+    def start(self, **kwargs) -> None:
         """Start the HTTP server and block until interrupted (Ctrl+C)."""
         chdir(self.directory)
+        self.logger.debug(f"Changing working directory to {self.directory}")
 
         # noinspection PyTypeChecker
         with ThreadingHTTPServer((self.host, self.port), self._handler_factory) as httpd:
+            # self._httpd seems to only be used by the close method
             self._httpd = httpd
-            self.logger.info(f"EasyHTTPServerAJM v{__version__}", print_msg=True)
-            # noinspection HttpUrlsUsage
-            self.logger.info(f"Serving directory {self.directory.resolve()} at http://{self.host}:{self.port}",
-                             print_msg=True)
-            print("Press Ctrl+C to stop.")
+
+            self._log_all_basic_server_info(**kwargs)
+
             try:
                 self.start_time = datetime.now()
+                self.logger.info(f"Server started at {self.start_time}", print_msg=True)
                 httpd.serve_forever()
             except KeyboardInterrupt:
                 self.logger.warning(f"Shutting down server (ran for {self.runtime}).")
@@ -119,8 +185,16 @@ class EasyHTTPServer:
             self._httpd.server_close()
             self._httpd = None
 
+    def err_stop(self) -> None:
+        """
+        Stop the server if it's running.
+        (Only useful if you manage the server in a separate thread/process.)
+        """
+        self.stop()
+        # FIXME: this error exit seems to be ignored - create my own ThreadingHTTPServer class and override shutdown?
+        exit(1)
+
 
 if __name__ == "__main__":
-    # TODO: make both CLI and class-based usage possible
     srv = EasyHTTPServer()
     srv.start()
